/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { getRandomString } from '@kbn/test-jest-helpers';
import { ClientWatchStatusModel } from '../common/types';
import { WATCH_STATES, WATCH_STATE_COMMENTS } from '../common/constants';

interface Watch {
  id: string;
  name: string;
  type: 'json' | 'threshold' | 'monitoring';
  timeField?: string;
  triggerIntervalSize?: number;
  triggerIntervalUnit?: string;
  aggType?: string;
  termSize?: number;
  thresholdComparator?: string;
  timeWindowSize?: number;
  timeWindowUnit?: string;
  threshold?: number[];
  isSystemWatch: boolean;
  watchStatus: ClientWatchStatusModel;
}

export const getWatch = ({
  id = getRandomString(),
  name = getRandomString(),
  type = 'json',
  timeField,
  triggerIntervalSize,
  triggerIntervalUnit,
  aggType,
  termSize,
  thresholdComparator,
  timeWindowSize,
  timeWindowUnit,
  threshold,
  isSystemWatch = false,
  watchStatus = {
    id: 'a',
    state: WATCH_STATES.OK,
    isActive: true,
    lastChecked: null,
    lastMetCondition: null,
    comment: WATCH_STATE_COMMENTS.OK,
    actionStatuses: [],
  },
}: Partial<Watch> = {}): Watch => ({
  id,
  name,
  type,
  timeField,
  triggerIntervalSize,
  triggerIntervalUnit,
  aggType,
  termSize,
  thresholdComparator,
  timeWindowSize,
  timeWindowUnit,
  threshold,
  isSystemWatch,
  watchStatus,
});
